/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createPostfachUriByVorgangId } from 'apps/alfa-e2e/src/support/postfach-util';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { EingangE2E, VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { PostfachMailPage } from '../../../page-objects/postfach-mail.component.po';
import { dropCollections, visitUrl } from '../../../support/cypress-helper';
import { contains, exist, notExist } from '../../../support/cypress.util';
import { ORGANISATIONSEINHEITEN_ID_FOR_KFINDER, ORGANISATIONSEINHEITEN_ID_FOR_KORDNER } from '../../../support/data.util';
import { MessagesE2E } from '../../../support/messages';
import { loginAsKfinder, loginAsKordner } from '../../../support/user-util';
import { buildVorgang, createVorgang, initVorgaenge } from '../../../support/vorgang-util';

describe('PostfachNachrichten filtered by organisationseinheit', () => {

	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

	const postfachMailPage: PostfachMailPage = new PostfachMailPage();

	const vorgangIdForKFinder: string = '601bc78eabc134051570aab8';
	const vorgangUrlVisibleToKFinder: string = `${createPostfachUriByVorgangId(vorgangIdForKFinder)}`;

	const eingangForKFinder: EingangE2E = { ...createVorgang().eingangs[0], zustaendigeStelle: { organisationseinheitenId: ORGANISATIONSEINHEITEN_ID_FOR_KFINDER } };
	const vorgangForKFinder: VorgangE2E = { ...buildVorgang(vorgangIdForKFinder, 'VorgangVisibleToKFinder'), eingangs: [eingangForKFinder] };

	const vorgangIdForKOrdner: string = '60250b9d383c182943f6ba79';
	const vorgangUrlVisibleToKOrdner: string = `${createPostfachUriByVorgangId(vorgangIdForKOrdner)}`;

	const eingangForKOrdner: EingangE2E = { ...createVorgang().eingangs[0], zustaendigeStelle: { organisationseinheitenId: ORGANISATIONSEINHEITEN_ID_FOR_KORDNER } };
	const vorgangForKOrdner: VorgangE2E = { ...buildVorgang(vorgangIdForKOrdner, 'VorgangVisibleToKOrdner'), eingangs: [eingangForKOrdner] };

	before(() => {
		initVorgaenge([vorgangForKFinder, vorgangForKOrdner]);
	})

	after(() => {
		dropCollections();
	})

	describe('on user kfinder', () => {
		const authorizedUrl = vorgangUrlVisibleToKFinder;
		const forbiddenUrl = vorgangUrlVisibleToKOrdner;

		before(() => {
			loginAsKfinder();

			waitForSpinnerToDisappear();
			exist(vorgangList.getRoot());
		})

		it('should open postfachNachrichten page', () => {
			logInParts(authorizedUrl);
			visitUrl(authorizedUrl);
			waitForSpinnerToDisappear();

			exist(postfachMailPage.getBreadcrump());
		});

		it('should not open postfachNachrichten page', () => {
			logInParts(forbiddenUrl);
			visitUrl(forbiddenUrl);
			waitForSpinnerToDisappear();

			notExist(postfachMailPage.getBreadcrump());
		})

		it('should show snackbar', () => {
			exist(snackbar.getCloseButton());
			contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
		})
	})

	describe('on user kordner', () => {
		const authorizedUrl = vorgangUrlVisibleToKOrdner;
		const forbiddenUrl = vorgangUrlVisibleToKFinder;

		before(() => {
			loginAsKordner();

			waitForSpinnerToDisappear();
			exist(vorgangList.getRoot());
		})

		it('should open postfachNachrichten page', () => {
			logInParts(authorizedUrl);
			visitUrl(authorizedUrl);
			waitForSpinnerToDisappear();

			exist(postfachMailPage.getBreadcrump())
		})

		it('should not open postfachNachrichten page', () => {
			logInParts(forbiddenUrl);
			visitUrl(forbiddenUrl);
			waitForSpinnerToDisappear();

			notExist(postfachMailPage.getBreadcrump())
		})

		it('should show snackbar', () => {
			exist(snackbar.getCloseButton());
			contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
		})
	})

	function logInParts(toLog: string): void {
		const step: number = 10;
		for (let i = 0; i < toLog.length;) {
			const subUrl: string = toLog.slice(i, i + step);
			Cypress.log({ name: `u.p.: ${subUrl}` });
			i += step;
		}
	}
});