/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { getUrl } from '@ngxp/rest';
import { hot } from 'jest-marbles';
import { of } from 'rxjs';
import { createBinaryFileResource } from '../../../binary-file-shared/test/binary-file';
import { createApiRoot, createApiRootResource } from '../../test/api-root';
import { ApiRootFacade } from './+state/api-root.facade';
import { ApiRoot, ApiRootResource } from './api-root.model';
import { ApiRootRepository } from './api-root.repository';
import { ApiRootService } from './api-root.service';

describe('ApiRootService', () => {

	var repository;
	var service;
	var facade;

	const apiRoot: ApiRootResource = createApiRootResource();
	const apiRootStateResource: StateResource<ApiRootResource> = createStateResource(apiRoot);

	beforeEach(() => {
		repository = mock(ApiRootRepository);
		facade = mock(ApiRootFacade);
		service = new ApiRootService(repository, facade);
	})

	describe('calling getApiRoot', () => {

		beforeEach(() => {
			repository.loadApiRoot.mockReturnValue(of(null));
		})

		it('should call repository', () => {
			service.getApiRoot();

			expect(repository.loadApiRoot).toHaveBeenCalled();
		});

		it('should not call repository if already loading', () => {
			service.apiRoot$.next(createEmptyStateResource(true));

			service.getApiRoot();

			expect(repository.loadApiRoot).not.toHaveBeenCalled();
		});

		it('should set loading flag', () => {
			service.getApiRoot();

			expect(service.apiRoot$.value.loading).toBe(true);
		});

		describe('on successful load', () => {
			const apiRoot: ApiRoot = createApiRoot();

			beforeEach(() => {
				repository.loadApiRoot.mockReturnValue(of(apiRoot));
			})

			it('should set loaded', () => {
				service.getApiRoot();

				expect(service.apiRoot$.value.loaded).toBe(true);
			});

			it('should set resource', () => {
				service.getApiRoot();

				expect(service.apiRoot$.value.resource).toBe(apiRoot);
			});

			it('should unset loading', () => {
				service.getApiRoot();

				expect(service.apiRoot$.value.loading).toBe(false);
			});
		})
	})

	describe('getDownloadToken', () => {

		const binaryFile: BinaryFileResource = createBinaryFileResource();

		beforeEach(() => {
			facade.getApiRoot.mockReturnValue(hot('-a', { a: apiRootStateResource }));
		})

		it('should call getApiRoot', () => {
			service.getDownloadToken(binaryFile);

			expect(facade.getApiRoot).toHaveBeenCalled();
		})

		it('should NOT call repository on null apiRoot', () => {
			service.getApiRoot = jest.fn().mockReturnValue(of(createEmptyStateResource()));

			service.getDownloadToken(binaryFile).subscribe();

			expect(repository.downloadToken).not.toHaveBeenCalled();
		})

		it('should call repository', () => {
			facade.getApiRoot.mockReturnValue(of(apiRootStateResource));

			service.getDownloadToken(binaryFile).subscribe();

			expect(repository.downloadToken).toHaveBeenCalledWith(apiRoot, getUrl(binaryFile));
		})
	})
})