/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse, HttpResponse } from '@angular/common/http';
import { faker } from '@faker-js/faker';
import { createEmptyStateResource, createStateResource, StateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { Resource } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { ValidationMessageCode, VALIDATION_MESSAGES } from 'libs/tech-shared/src/lib/validation/tech.validation.messages';
import { DummyLinkRel } from 'libs/tech-shared/test/dummy';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { of } from 'rxjs';
import { BinaryFileResource } from './binary-file.model';
import { BinaryFileRepository } from './binary-file.repository';
import { BinaryFileService } from './binary-file.service';

describe('BinaryFileService', () => {
	let service: BinaryFileService;

	let repository: Mock<BinaryFileRepository>
	let snackBarService: Mock<SnackBarService>;

	beforeEach(() => {
		repository = mock(BinaryFileRepository);
		snackBarService = mock(SnackBarService);

		service = new BinaryFileService(useFromMock(repository), useFromMock(snackBarService));
	})

	it('should created', () => {
		expect(service).toBeTruthy();
	})

	describe('download file', () => {

		const binaryFileResource: BinaryFileResource = createBinaryFileResource();
		const downloadNamePrefix: string = 'VorgangsNummerAsPrefix';

		const blob = {};

		beforeEach(() => {
			service.save = jest.fn();
			repository.download.mockReturnValue(hot('a', { a: blob }));
		})

		it('should call repository', () => {
			service.downloadFile(binaryFileResource, downloadNamePrefix);

			expect(repository.download).toHaveBeenCalledWith(binaryFileResource);
		})

		describe('save file', () => {

			describe('on existing response', () => {

				it('should save file without prefix', () => {
					service.saveBinaryFile(blob, binaryFileResource, undefined);

					expect(service.save).toHaveBeenCalledWith(blob, binaryFileResource.name);
				})

				it('should save file with prefix', () => {
					service.saveBinaryFile(blob, binaryFileResource, downloadNamePrefix);

					expect(service.save).toHaveBeenCalledWith(blob, downloadNamePrefix + '_' + binaryFileResource.name);
				})

				it('should return loaded stateResource', () => {
					const result: StateResource<Blob> = service.saveBinaryFile(blob, binaryFileResource, downloadNamePrefix);

					expect(result).toEqual(createStateResource(blob));
				})
			})

			describe('on non existing response', () => {

				it('should return loading stateResource', () => {
					const result: StateResource<Blob> = service.saveBinaryFile(undefined, binaryFileResource, downloadNamePrefix);

					expect(result).toEqual(createEmptyStateResource(true));
				})
			})
		})
	})

	describe('upload file', () => {

		const dummyResource: Resource = createDummyResource();
		const dummyLinkRel: string = DummyLinkRel.DUMMY;

		const fileLocation: string = 'fileLocation';
		const uploadFileResponse: HttpResponse<void> = <any>{ response: { headers: { ['Location']: fileLocation } } };

		const testFile: any = {};
		const returnFile: any = {};

		beforeEach(() => {
			repository.uploadFile.mockReturnValue(hot('-a', { a: uploadFileResponse }));
			repository.getFile.mockReturnValue(of(returnFile));
		})

		it('should call repository', () => {
			service.uploadFile(dummyResource, dummyLinkRel, testFile)

			expect(repository.uploadFile).toHaveBeenCalledWith(dummyResource, dummyLinkRel, testFile);
		})

		it.skip('FIXME: should call get file', () => {
			repository.uploadFile.mockReturnValue(of(uploadFileResponse));
			repository.getFile.mockReturnValue(of(returnFile));

			service.uploadFile(dummyResource, dummyLinkRel, testFile);

			expect(repository.getFile).toHaveBeenCalledWith(fileLocation);
		})

		describe('handleErrorByStaus', () => {

			it('should show snackbar on ' + ValidationMessageCode.VALIDATION_FIELD_FILE_SIZE_EXCEEDED + ' error', () => {
				const response: HttpErrorResponse = buildErrorResponse(ValidationMessageCode.VALIDATION_FIELD_FILE_SIZE_EXCEEDED);

				service.handleErrorByStatus(response);

				expect(snackBarService.showError).toHaveBeenCalledWith(VALIDATION_MESSAGES[ValidationMessageCode.VALIDATION_FIELD_FILE_SIZE_EXCEEDED]);
			})

			it('should not call snackbar on other error', () => {
				const response: HttpErrorResponse = buildErrorResponse(ValidationMessageCode.VALIDATION_FIELD_EMPTY);

				service.handleErrorByStatus(response);

				expect(snackBarService.showError).not.toHaveBeenCalled();
			})

			function buildErrorResponse(messageCode: string): HttpErrorResponse {
				return <HttpErrorResponse>{ status: 422, error: { issues: [{ messageCode, parameters: [] }] } }
			}
		})
	})

	describe('get file', () => {

		const responseResource: Resource = createDummyResource();

		const uri: string = faker.internet.url();

		beforeEach(() => {
			repository.getFile.mockReturnValue(hot('-a', { a: responseResource }));
		})

		it('should call repository', () => {
			service.getFile(uri);

			expect(repository.getFile).toHaveBeenCalledWith(uri);
		})

		it('should return value', () => {
			const fileList = service.getFile(uri);

			expect(fileList).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: createStateResource(responseResource) }));
		})
	})
})