/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { PostfachMail } from '@alfa-client/postfach-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createPostfachMail, createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { ExpansionPanelWithUserComponent } from '../expansion-panel-with-user/expansion-panel-with-user.component';
import { HistorieItemAttachmentComponent } from '../historie-item-attachment/historie-item-attachment.component';
import { HistorieItemHeaderComponent } from '../historie-item-header/historie-item-header.component';
import { HistorieItemPostfachNachrichtComponent } from './historie-item-postfach-nachricht.component';

describe('HistorieItemPostfachNachrichtComponent', () => {
	let component: HistorieItemPostfachNachrichtComponent;
	let fixture: ComponentFixture<HistorieItemPostfachNachrichtComponent>;

	const item: PostfachMail = createPostfachMail();
	const postfachNachrichtCommand: CommandResource = { ...createCommandResource(), body: { item: item } };

	const attachment: string = getDataTestClassOf('historie-item-postfach-nachricht-attachment');
	const expansionPanel: string = getDataTestClassOf('historie-item-postfach-nachricht-user-expansion-panel');
	const mailBody: string = getDataTestClassOf('postfach-nachricht-mail-body');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				HistorieItemPostfachNachrichtComponent,
				MockComponent(ExpansionPanelWithUserComponent),
				MockComponent(HistorieItemAttachmentComponent),
				MockComponent(HistorieItemHeaderComponent)
			],
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(
			HistorieItemPostfachNachrichtComponent
		);
		component = fixture.componentInstance;
		component.command = postfachNachrichtCommand;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnInit', () => {

		it('should set headline for ' + CommandOrder.SEND_POSTFACH_NACHRICHT, () => {
			component.command = { ...postfachNachrichtCommand, order: CommandOrder.SEND_POSTFACH_NACHRICHT };

			component.ngOnInit();

			expect(component.headline).toBe('eine Nachricht geschrieben.');
		})

		it('should set headline by order' + CommandOrder.RECEIVE_POSTFACH_NACHRICHT, () => {
			component.command = { ...postfachNachrichtCommand, order: CommandOrder.RECEIVE_POSTFACH_NACHRICHT };

			component.ngOnInit();

			expect(component.headline).toBe('eine Nachricht des Antragstellers empfangen.');
		})

		it('should set postfach nachricht', () => {
			component.getPostfachNachricht = jest.fn();

			component.ngOnInit();

			expect(component.getPostfachNachricht).toHaveBeenCalled();
		})

		describe('get postfach nachricht', () => {

			it('should return body if order is SEND_POSTFACH_NACHRICHT', () => {
				component.command = { ...postfachNachrichtCommand, order: CommandOrder.SEND_POSTFACH_NACHRICHT };
				fixture.detectChanges();

				const postfachNachricht = component.getPostfachNachricht();

				expect(postfachNachricht).toEqual({ item });
			})

			it('should return item in body if order is NOT SEND_POSTFACH_NACHRICHT', () => {
				component.command = { ...postfachNachrichtCommand, order: CommandOrder.RECEIVE_POSTFACH_NACHRICHT };
				fixture.detectChanges();

				const postfachNachricht = component.getPostfachNachricht();

				expect(postfachNachricht).toEqual(item);
			})
		})
	})

	describe('attachments', () => {

		it('should hide of NOT exists', () => {
			const item: PostfachMail = createPostfachMail();
			delete item['attachments'];
			component.postfachNachricht = item;
			fixture.detectChanges();

			const attachmentElement = getElementFromFixture(fixture, attachment);

			expect(attachmentElement).not.toBeInstanceOf(HTMLElement);
		})

		it('should show if exists', () => {
			component.postfachNachricht = createPostfachMail();
			fixture.detectChanges();

			const attachmentElement = getElementFromFixture(fixture, attachment);

			expect(attachmentElement).toBeInstanceOf(HTMLElement);
		})
	})

	describe('postfach nachricht body exists', () => {

		beforeEach(() => {
			component.postfachNachricht = createPostfachMailResource();
			fixture.detectChanges();
		})

		it('should show postfach nachricht with body', () => {
			const element = getElementFromFixture(fixture, mailBody);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should hide postfach nachricht', () => {
			const element = getElementFromFixture(fixture, expansionPanel);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})


	describe('postfach nachricht body not exists', () => {

		beforeEach(() => {
			component.postfachNachricht = undefined;
			fixture.detectChanges();
		})

		it('should show postfach nachricht', () => {
			const element = getElementFromFixture(fixture, expansionPanel);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should hide postfach nachricht with body', () => {
			const element = getElementFromFixture(fixture, mailBody);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})
});
