/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TestBed } from '@angular/core/testing';
import { ApiError, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { UserProfileResource } from '@alfa-client/user-profile-shared';
import { provideMockActions } from '@ngrx/effects/testing';
import { Action } from '@ngrx/store';
import { provideMockStore } from '@ngrx/store/testing';
import { cold, hot } from 'jest-marbles';
import { createApiError } from 'libs/tech-shared/test/error';
import { createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import { createUserSettings, createUserSettingsResource } from 'libs/user-settings-shared/test/user-settings';
import { Observable, of } from 'rxjs';
import { UserSettings, UserSettingsResource } from '../user-settings.model';
import { UserSettingsRepository } from '../user-settings.repository';
import * as UserSettingsActions from './user-settings.actions';
import { UserSettingsEffects } from './user-settings.effects';
import { UserSettingsFacade } from './user-settings.facade';

describe('UserSettingsEffects', () => {
	let actions: Observable<Action>;
	let effects: UserSettingsEffects;

	const userSettingsFacade: Mock<UserSettingsFacade> = mock(UserSettingsFacade);
	const userSettingsRepository: Mock<UserSettingsRepository> = mock(UserSettingsRepository);

	beforeEach(() => {
		TestBed.configureTestingModule({
			providers: [
				UserSettingsEffects,
				provideMockActions(() => actions),
				provideMockStore(),
				{
					provide: UserSettingsRepository,
					useValue: userSettingsRepository
				},
				{
					provide: UserSettingsFacade,
					useValue: userSettingsFacade
				}
			],
		});

		effects = TestBed.inject(UserSettingsEffects);
	});

	describe('loadUserSettings', () => {
		const currentUser: UserProfileResource = createUserProfileResource();
		const action = UserSettingsActions.loadUserSettings({ currentUser });
		const userSettings: UserSettingsResource = createUserSettingsResource();

		it('should call repository', () => {
			actions = of(action);

			effects.loadUserSettings$.subscribe();

			expect(userSettingsRepository.getUserSettings).toHaveBeenCalledWith(currentUser);
		})

		it('should dispatch success action', () => {
			userSettingsRepository.getUserSettings.mockReturnValue(of(userSettings));

			actions = hot('-a', { a: action });

			const expected = cold('-b', { b: UserSettingsActions.loadUserSettingsSuccess({ userSettings }) });
			expect(effects.loadUserSettings$).toBeObservable(expected);
		})

		it('should dispatch error action', () => {
			const apiError: ApiError = createApiError()
			const error = { error: { error: apiError } };
			const errorResponse = cold('-#', {}, error);
			userSettingsRepository.getUserSettings = jest.fn(() => errorResponse);

			const expected = cold('--b', { b: UserSettingsActions.loadUserSettingsFailure({ apiError }) });
			actions = hot('-a', { a: action });

			expect(effects.loadUserSettings$).toBeObservable(expected);
		})
	})

	describe('set user settings', () => {

		const userSettingsResource: UserSettingsResource = createUserSettingsResource();
		const userSettings: UserSettings = createUserSettings();
		const action = UserSettingsActions.setUserSettings({ userSettings });

		beforeEach(() => {
			userSettingsFacade.getUserSettings.mockReturnValue(of(createStateResource(userSettingsResource)));
			userSettingsRepository.setUserSettings.mockReturnValue(of(userSettingsResource));
		})

		it('should call repository', () => {
			actions = of(action);

			effects.setUserSettings$.subscribe();

			expect(userSettingsRepository.setUserSettings).toHaveBeenCalledWith(userSettingsResource, userSettings);
		})

		it('should dispatch success action', () => {
			userSettingsRepository.setUserSettings.mockReturnValue(of(userSettingsResource));

			actions = hot('-a-|', { a: action });

			const expected = cold('-a-|', { a: UserSettingsActions.setUserSettingsSuccess({ userSettings: userSettingsResource }) });
			expect(effects.setUserSettings$).toBeObservable(expected);
		})

		it('should dispatch error action', () => {
			const apiError: ApiError = createApiError()
			const error = { error: { error: apiError } };
			const errorResponse = cold('-#', {}, error);
			userSettingsRepository.setUserSettings = jest.fn(() => errorResponse);

			const expected = cold('--b', { b: UserSettingsActions.setUserSettingsFailure({ apiError }) });
			actions = hot('-a', { a: action });

			expect(effects.setUserSettings$).toBeObservable(expected);
		})
	})
});
