/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.errorhandling;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Stream;

import org.hibernate.validator.engine.HibernateConstraintViolation;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import de.ozgcloud.alfa.common.binaryfile.DynamicViolationParameter;
import de.ozgcloud.common.errorhandling.ExceptionUtil;
import de.ozgcloud.common.errorhandling.TechnicalException;
import jakarta.validation.ConstraintViolation;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Path;
import jakarta.validation.metadata.ConstraintDescriptor;
import lombok.extern.log4j.Log4j2;

@ControllerAdvice
@Log4j2
@Order(98)
public class ExceptionController {

	private static final Set<String> IGNORABLE_CONSTRAINT_VIOLATION_ATTRIBUTES = new HashSet<>(Arrays.asList("groups", "payload", "message"));

	static final String RUNTIME_MESSAGE_CODE = "generale.server_error";
	static final String RESOURCE_NOT_FOUNT_MESSAGE_CODE = "resource.not_found";
	static final String ACCESS_DENIED_MESSAGE_CODE = "generale.access_denied";

	@ExceptionHandler(FunctionalException.class)
	@ResponseStatus(HttpStatus.BAD_REQUEST)
	@ResponseBody
	public ApiError handleFunctionalException(FunctionalException e) {
		LOG.error("BadRequest", e);
		return ApiError.builder().issue(buildIssueForFunctionalException(e)).build();
	}

	private Issue buildIssueForFunctionalException(FunctionalException e) {
		return Issue.builder().message(e.getMessage()).messageCode(e.getErrorCode()).exceptionId(e.getExceptionId()).build();
	}

	@ExceptionHandler(AccessDeniedException.class)
	@ResponseStatus(HttpStatus.FORBIDDEN)
	@ResponseBody
	public ApiError handleAccessDeniedException(AccessDeniedException e) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error("Access Denied Exception", messageWithExceptionId);

		return ApiError.builder().issue(buildIssueForAccessDeniedException(messageWithExceptionId, exceptionId)).build();
	}

	private Issue buildIssueForAccessDeniedException(String message, String exceptionId) {
		return Issue.builder().messageCode(ACCESS_DENIED_MESSAGE_CODE).message(message).exceptionId(exceptionId).build();
	}

	@ExceptionHandler(ResourceNotFoundException.class)
	@ResponseStatus(HttpStatus.NOT_FOUND)
	@ResponseBody
	public ApiError handleResourceNotFoundException(ResourceNotFoundException e) {
		LOG.warn("Resource not found: {}", e.getMessage());
		return ApiError.builder().issue(buildIssueForResourceNotFoundException(e)).build();
	}

	private Issue buildIssueForResourceNotFoundException(ResourceNotFoundException e) {
		return Issue.builder().message(e.getMessage()).messageCode(RESOURCE_NOT_FOUNT_MESSAGE_CODE).exceptionId(e.getExceptionId()).build();
	}

	@ExceptionHandler(ConstraintViolationException.class)
	@ResponseStatus(HttpStatus.UNPROCESSABLE_ENTITY)
	@ResponseBody
	public ApiError handleConstraintViolationException(ConstraintViolationException e) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.warn("Validation Exception: {}", messageWithExceptionId);
		return ApiError.builder().issues(buildIssues(e, exceptionId)).build();
	}

	private List<Issue> buildIssues(ConstraintViolationException e, String exceptionId) {
		return e.getConstraintViolations().stream()
				.map(violation -> buildIssue(violation, exceptionId))
				.toList();
	}

	private Issue buildIssue(ConstraintViolation<?> violation, String exceptionId) {
		return Issue.builder()//
				.field(buildFieldPath(violation.getPropertyPath()))//
				.messageCode(violation.getMessageTemplate().replace("{", "").replace("}", ""))//
				.message(violation.getMessage())//
				.parameters(buildParameters(violation).toList())
				.exceptionId(exceptionId)
				.build();
	}

	private String buildFieldPath(Path propertyPath) {
		return propertyPath.toString().substring(propertyPath.toString().indexOf('.') + 1);
	}

	Stream<IssueParam> buildParameters(ConstraintViolation<?> violation) {
		var dynamicPayload = getDynamicPayload(violation);
		return Optional.ofNullable(violation.getConstraintDescriptor())
				.map(ConstraintDescriptor::getAttributes)
				.map(descr -> descr.entrySet().stream()
						.filter(entry -> !IGNORABLE_CONSTRAINT_VIOLATION_ATTRIBUTES.contains(entry.getKey()))
						.map(entryMap -> buildIssueParam(entryMap, dynamicPayload)))
				.orElse(Stream.empty());
	}

	private IssueParam buildIssueParam(Entry<String, Object> entry, Optional<DynamicViolationParameter> dynamicValues) {
		return IssueParam.builder().name(entry.getKey()).value(getValue(entry, dynamicValues)).build();
	}

	private String getValue(Entry<String, Object> entry, Optional<DynamicViolationParameter> dynamicValues) {
		return dynamicValues
				.map(DynamicViolationParameter::getMap)
				.map(map -> map.get(entry.getKey()))
				.filter(Objects::nonNull)
				.map(String::valueOf)
				.orElse(String.valueOf(entry.getValue()));
	}

	private Optional<DynamicViolationParameter> getDynamicPayload(ConstraintViolation<?> violation) {
		HibernateConstraintViolation<?> hibernateViolation = violation.unwrap(HibernateConstraintViolation.class);
		return Optional.ofNullable(hibernateViolation.getDynamicPayload(DynamicViolationParameter.class));
	}

	@ExceptionHandler(TechnicalException.class)
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	@ResponseBody
	public ApiError handleTechnicalException(TechnicalException e) {
		LOG.error("TechnicalException on Request", e);
		return buildRuntimeApiError(e.getMessage(), e.getExceptionId());
	}

	@ExceptionHandler(RuntimeException.class)
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	@ResponseBody
	public ApiError handleRuntimeException(RuntimeException e) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error("RuntimeException on Request: " + messageWithExceptionId, e);
		return buildRuntimeApiError(messageWithExceptionId, exceptionId);
	}

	String createExceptionId() {
		return UUID.randomUUID().toString();
	}

	private ApiError buildRuntimeApiError(String message, String exceptionId) {
		return ApiError.builder().issue(createIssueForRuntimeException(message, exceptionId)).build();
	}

	private Issue createIssueForRuntimeException(String message, String exceptionId) {
		return Issue.builder().messageCode(RUNTIME_MESSAGE_CODE).message(message).exceptionId(exceptionId).build();
	}
}