/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Optional;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcAccessPermission;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttribute;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcDeleteClientAttributeRequest;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcSetClientAttributeRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
public class WiedervorlageRemoteService {

	static final String CLIENT_ATTRIBUTE_NEXT_WIEDERVORLAGE_FRIST = "nextWiedervorlageFrist";
	static final String ITEM_NAME = "Wiedervorlage";

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private VorgangAttachedItemServiceBlockingStub vorgangAttachedItemServiceStub;

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private ClientAttributeServiceBlockingStub clientAttributeServiceStub;

	@Autowired
	private WiedervorlageMapper mapper;
	@Autowired
	private ApplicationContext context;

	public Stream<Wiedervorlage> findByVorgangId(String vorgangId) {
		var response = vorgangAttachedItemServiceStub.find(buildFindRequest(vorgangId));

		return response.getVorgangAttachedItemsList().stream().map(mapper::fromItem);
	}

	GrpcFindVorgangAttachedItemRequest buildFindRequest(String vorgangId) {
		return GrpcFindVorgangAttachedItemRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setItemName(ITEM_NAME)
				.build();
	}

	public Wiedervorlage getById(String wiedervorlageId) {
		var response = vorgangAttachedItemServiceStub.getById(GrpcVorgangAttachedItemRequest.newBuilder().setId(wiedervorlageId).build());

		return mapper.fromItem(response.getVorgangAttachedItem()).toBuilder().vorgangId(response.getVorgangAttachedItem().getVorgangId()).build();
	}

	public void updateNextFrist(String vorgangId, Optional<LocalDate> nextFrist) {
		nextFrist.ifPresentOrElse(frist -> setNextFrist(vorgangId, frist), () -> deleteNextFrist(vorgangId));
	}

	void deleteNextFrist(String vorgangId) {
		clientAttributeServiceStub.delete(GrpcDeleteClientAttributeRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setClientName(context.getId())
				.setAttributeName(CLIENT_ATTRIBUTE_NEXT_WIEDERVORLAGE_FRIST)
				.build());
	}

	void setNextFrist(String vorgangId, LocalDate nextFrist) {
		clientAttributeServiceStub.set(buildUpdateNextFristRequest(vorgangId, nextFrist));
	}

	private GrpcSetClientAttributeRequest buildUpdateNextFristRequest(String vorgangId, LocalDate nextFrist) {
		return GrpcSetClientAttributeRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setAttribute(GrpcClientAttribute.newBuilder()
						.setAccess(GrpcAccessPermission.READ_ONLY)
						.setAttributeName(CLIENT_ATTRIBUTE_NEXT_WIEDERVORLAGE_FRIST)
						.setValue(GrpcClientAttributeValue.newBuilder().setStringValue(nextFrist.format(DateTimeFormatter.ISO_DATE)))
						.build())
				.build();
	}
}