/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static de.ozgcloud.alfa.common.command.CommandModelAssembler.*;
import static de.ozgcloud.alfa.common.command.CommandTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.InjectMocks;
import org.springframework.context.ApplicationContext;
import org.springframework.core.env.Environment;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class CommandModelAssemblerTest {

	private static final String COMMAND_SINGLE_PATH = "/api/commands/" + CommandTestFactory.ID;

	@InjectMocks
	private CommandModelAssembler modelAssembler;

	@BeforeEach
	void init() {
		var applicationContext = mock(ApplicationContext.class);
		lenient().when(applicationContext.getEnvironment()).thenReturn(mock(Environment.class));
		new UserProfileUrlProvider().setApplicationContext(applicationContext);
	}

	@Test
	void shouldHaveSelfLink() {
		var model = modelAssembler.toModel(CommandTestFactory.create());

		assertThat(model.getLink(IanaLinkRelations.SELF)).isPresent().get().extracting(Link::getHref).isEqualTo(COMMAND_SINGLE_PATH);
	}

	@Nested
	@DisplayName("Link to effected Resource")
	class TestEffectedResourceLink {

		@DisplayName("on vorgang")
		@Nested
		class TestOnVorgang {

			private final String VORGANG_URL = "/api/vorgangs/" + RELATION_ID;

			@ParameterizedTest
			@EnumSource(mode = Mode.INCLUDE, names = { "FINISHED", "REVOKED" })
			void shouldBePresentOnStatus(CommandStatus status) {
				var model = toModelWithStatus(status);

				assertThat(model.getLink(REL_EFFECTED_RESOURCE)).isPresent().get().extracting(Link::getHref).isEqualTo(VORGANG_URL);
			}

			@ParameterizedTest
			@EnumSource(mode = Mode.EXCLUDE, names = { "FINISHED", "REVOKED", "ERROR" })
			void shouldNOTbePresentOtherWise(CommandStatus status) {
				var model = toModelWithStatus(status);

				assertThat(model.getLink(REL_EFFECTED_RESOURCE)).isEmpty();
			}

			private EntityModel<Command> toModelWithStatus(CommandStatus status) {
				return modelAssembler.toModel(CommandTestFactory.createBuilder().status(status).build());
			}
		}

		@DisplayName("on wiedervorlage")
		@Nested
		class TestOnWiedervorlage {

			private final String WIEDERVORLAGE_URL = "/api/wiedervorlages/" + RELATION_ID;

			@ParameterizedTest
			@EnumSource(mode = Mode.INCLUDE, names = { "CREATE_WIEDERVORLAGE", "EDIT_WIEDERVORLAGE",
					"WIEDERVORLAGE_ERLEDIGEN", "WIEDERVORLAGE_WIEDEREROEFFNEN" })
			void shouldHaveLinkToWiedervorlage(CommandOrder order) {
				var model = toModelWithOrder(order);

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isPresent().get().extracting(Link::getHref)
						.isEqualTo(WIEDERVORLAGE_URL);
			}
		}

		@DisplayName("on kommentar")
		@Nested
		class TestOnKommentar {

			private final String KOMMENTAR_URL = "/api/kommentars/" + CommandTestFactory.RELATION_ID;

			@ParameterizedTest
			@EnumSource(mode = Mode.INCLUDE, names = { "CREATE_KOMMENTAR", "EDIT_KOMMENTAR" })
			void shoulHaveLinkToKommentar(CommandOrder order) {
				var model = toModelWithOrder(order);

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isPresent().get()
						.extracting(Link::getHref).isEqualTo(KOMMENTAR_URL);
			}
		}

		@DisplayName("on forwarding")
		@Nested
		class TestOnForwarding {

			private final String FORWARDING_URL = "/api/forwardings?vorgangId=" + VorgangHeaderTestFactory.ID;

			@ParameterizedTest
			@EnumSource(mode = Mode.INCLUDE, names = { "FORWARD_SUCCESSFULL", "FORWARD_FAILED", "REDIRECT_VORGANG" })
			void shouldHaveLinkToForwarding(CommandOrder order) {
				var model = toModelWithOrder(order);

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isPresent().get()
						.extracting(Link::getHref).isEqualTo(FORWARDING_URL);
			}
		}

		@DisplayName("on postfach")
		@Nested
		class TestOnPostfach {

			private final String POSTFACH_URL = "/api/postfachMails?vorgangId=" + VorgangHeaderTestFactory.ID;

			@ParameterizedTest
			@EnumSource(mode = Mode.INCLUDE, names = { "SEND_POSTFACH_MAIL" })
			void shouldHaveLinkToPostfach(CommandOrder order) {
				var model = toModelWithOrder(order);

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isPresent().get()
						.extracting(Link::getHref).isEqualTo(POSTFACH_URL);
			}
		}

		private EntityModel<Command> toModelWithOrder(CommandOrder order) {
			return modelAssembler.toModel(CommandTestFactory.createBuilder().order(order).build());
		}

		@ParameterizedTest
		@EnumSource
		void shouldBeAbleToBuildLinkForEveryOrder(CommandOrder order) {
			var link = modelAssembler.effectedResourceLinkByOrderType(CommandTestFactory.createBuilder().order(order).build());

			assertThat(link).isNotNull();
		}

		@Test
		void shouldAddLinkForTypeVORGANG_LIST() {
			var link = modelAssembler
					.effectedResourceLinkByOrderType(CommandTestFactory.createBuilder().order(CommandOrder.VORGANG_LOESCHEN).build());

			assertThat(link.getHref()).isEqualTo("/api/vorgangs");
		}
	}

	@DisplayName("Revoke link")
	@Nested
	class TestRevokeLink {

		@DisplayName("related to command")
		@Nested
		class TestRelatedToCommand {

			@ParameterizedTest
			@EnumSource(mode = Mode.EXCLUDE, names = { "CREATE_WIEDERVORLAGE", "WIEDERVORLAGE_ERLEDIGEN", "EDIT_WIEDERVORLAGE",
					"WIEDERVORLAGE_WIEDEREROEFFNEN", "CREATE_KOMMENTAR", "EDIT_KOMMENTAR", "REDIRECT_VORGANG", "FORWARD_SUCCESSFULL",
					"FORWARD_FAILED", "ASSIGN_USER", "SEND_POSTFACH_MAIL", "SEND_POSTFACH_NACHRICHT", "RESEND_POSTFACH_MAIL", "CREATE_ATTACHED_ITEM",
					"UPDATE_ATTACHED_ITEM", "PATCH_ATTACHED_ITEM", "RECEIVE_POSTFACH_NACHRICHT", "VORGANG_LOESCHEN", "DELETE_ATTACHED_ITEM",
					"VORGANG_ZUM_LOESCHEN_MARKIEREN", "LOESCH_ANFORDERUNG_ZURUECKNEHMEN", "CREATE_BESCHEID", "PROCESS_VORGANG", "SET_AKTENZEICHEN" })
			void shouldBePresentOnOrder(CommandOrder order) {
				var model = toModelWithOrder(order);

				assertThat(model.getLink(REL_REVOKE)).isPresent().get().extracting(Link::getHref).isEqualTo(COMMAND_SINGLE_PATH);
			}
		}

		@DisplayName("related to loeschAnforderung")
		@Nested
		class TestRelatedToLoeschAnforderung {

			@ParameterizedTest
			@EnumSource(names = { "VORGANG_ZUM_LOESCHEN_MARKIEREN", "LOESCH_ANFORDERUNG_ZURUECKNEHMEN" })
			void shouldNotBePresentOnOrder(CommandOrder order) {
				var model = toModelWithOrder(order);

				assertThat(model.getLink(REL_REVOKE)).isNotPresent();
			}
		}

		@ParameterizedTest
		@EnumSource(names = { "CREATE_WIEDERVORLAGE", "WIEDERVORLAGE_ERLEDIGEN", "EDIT_WIEDERVORLAGE",
				"WIEDERVORLAGE_WIEDEREROEFFNEN", "CREATE_KOMMENTAR", "EDIT_KOMMENTAR", "REDIRECT_VORGANG", "FORWARD_SUCCESSFULL" })
		void shouldNOTBePresentOnOrder(CommandOrder order) {
			var model = toModelWithOrder(order);

			assertThat(model.getLink(REL_REVOKE)).isNotPresent();
		}

		@Test
		void shouldNotHaveLinkIfCommandFailed() {
			var command = CommandTestFactory.createBuilder().order(CommandOrder.VORGANG_ANNEHMEN).status(CommandStatus.ERROR).build();
			var model = modelAssembler.toModel(command);

			assertThat(model.getLink(REL_REVOKE)).isNotPresent();
		}

		private EntityModel<Command> toModelWithOrder(CommandOrder order) {
			return modelAssembler.toModel(CommandTestFactory.createBuilder().order(order).build());
		}
	}

	@DisplayName("Update link")
	@Nested
	class TestUpdateLink {

		@ParameterizedTest
		@EnumSource(names = { "PENDING", "REVOKE_PENDING" })
		void shouldHave(CommandStatus status) {
			var model = modelAssembler.toModel(CommandTestFactory.createBuilder().status(status).build());

			assertThat(model.getLink(REL_UPDATE)).isPresent().get().extracting(Link::getHref).isEqualTo(COMMAND_SINGLE_PATH);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "PENDING", "REVOKE_PENDING" })
		void shouldNotHaveLink(CommandStatus status) {
			var model = modelAssembler.toModel(CommandTestFactory.createBuilder().status(status).build());

			assertThat(model.getLink(REL_UPDATE)).isNotPresent();
		}
	}
}