/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.RepresentationModel;

import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.postfach.PostfachMail.Direction;
import de.ozgcloud.alfa.postfach.PostfachMailController.PostfachMailCommandByVorgangController;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangHeadTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class PostfachMailModelAssemblerTest {

	@InjectMocks
	private PostfachMailModelAssembler modelAssembler;
	@Mock
	private UserManagerUrlProvider userManagerUrlProvider;

	@BeforeEach
	void mock() {
		when(userManagerUrlProvider.isConfiguredForUserProfile()).thenReturn(false);
	}

	@Nested
	class TestLinksOnModel {

		@Test
		void shouldHaveSelfLink() {
			var link = modelAssembler.toModel(PostfachMailTestFactory.create()).getLink(IanaLinkRelations.SELF_VALUE);

			assertThat(link).isPresent().get().extracting(Link::getHref).isEqualTo(PostfachMailController.PATH + "/" + PostfachMailTestFactory.ID);
		}

		@Nested
		class ResendLink {

			@DisplayName("if 'sentSuccessful' is false, add link")
			@Test
			void shouldHaveResendLink() {
				var link = modelAssembler.toModel(PostfachMailTestFactory.createBuilder().sentSuccessful(false).build())
						.getLink(PostfachMailModelAssembler.REL_RESEND_POSTFACH_MAIL);

				assertThat(link).isPresent().get().extracting(Link::getHref)
						.isEqualTo("/api/postfachMails/" + PostfachMailTestFactory.ID + "/commands");
			}

			@Nested
			class ShouldNOTHaveResendLink {

				@DisplayName("if 'sentSuccessful' is true, hide link")
				@Test
				void ifSuccessfulSent() {
					var link = modelAssembler.toModel(PostfachMailTestFactory.createBuilder().sentSuccessful(true).build())
							.getLink(PostfachMailModelAssembler.REL_RESEND_POSTFACH_MAIL);

					assertThat(link).isNotPresent();
				}

				@Test
				void ifNotJetTriedToSend() {
					var nachricht = PostfachMailTestFactory.createBuilder().sentSuccessful(false).sentAt(null).build();

					var link = modelAssembler.toModel(nachricht).getLink(PostfachMailModelAssembler.REL_RESEND_POSTFACH_MAIL);

					assertThat(link).isNotPresent();
				}
			}

			@Nested
			class SendLink {
				@DisplayName("should be present if nachricht not jet tried to send")
				@Test
				void shouldBePresent() {
					var nachricht = PostfachMailTestFactory.createBuilder().sentSuccessful(false).sentAt(null).build();

					var link = modelAssembler.toModel(nachricht).getLink(PostfachMailModelAssembler.REL_SEND);

					assertThat(link).isPresent().get().extracting(Link::getHref)
							.isEqualTo("/api/postfachMails/" + PostfachMailTestFactory.ID + "/commands");
				}

				@Nested
				class ShouldNOTBePresent {
					@Test
					void ifNachrichtSent() {
						var nachricht = PostfachMailTestFactory.createBuilder().sentSuccessful(true).build();

						assertThat(modelAssembler.toModel(nachricht).getLink(PostfachMailModelAssembler.REL_SEND)).isNotPresent();
					}
				}
			}
		}

		@Nested
		class ResetHasNewPostfachNachricht {
			Stream<PostfachMail> mails = List.of(PostfachMailTestFactory.create()).stream();

			@Test
			void shouldHaveLink() {
				var link = modelAssembler.toCollectionModel(mails, PostfachTestFactory.create(), VorgangWithEingangTestFactory.create())
						.getLink(PostfachMailModelAssembler.REL_RESET_NEW_POSTFACH_MAIL);

				assertThat(link).isPresent();
			}

			@Test
			void shouldNotNaveLink() {
				var link = modelAssembler
						.toCollectionModel(mails, PostfachTestFactory.create(),
								VorgangWithEingangTestFactory.createBuilder().hasNewPostfachNachricht(false).build())
						.getLink(PostfachMailModelAssembler.REL_RESET_NEW_POSTFACH_MAIL);

				assertThat(link).isNotPresent();
			}
		}

		@Nested
		class ToAttachments {

			@Test
			@DisplayName("should be present if attachments present")
			void shouldBePresent() {
				var link = modelAssembler.toModel(PostfachMailTestFactory.create()).getLink(PostfachMailModelAssembler.REL_ATTACHMENTS);

				assertThat(link).isPresent().get().extracting(Link::getHref)
						.isEqualTo("/api/postfachMails/" + PostfachMailTestFactory.ID + "/attachments");
			}

			@Test
			@DisplayName("should NOT be present if there are no attachments")
			void shouldNOTBePresent() {
				var link = modelAssembler.toModel(PostfachMailTestFactory.createBuilder().clearAttachments().build())
						.getLink(PostfachMailModelAssembler.REL_ATTACHMENTS);

				assertThat(link).isNotPresent();
			}
		}

		@DisplayName("created by link")
		@Nested
		class CreatedByLink {

			@Nested
			class TestOnConfiguredUserManager {

				private final String userProfileTemplate = "UserProfileTemplateDummy/%s";

				@BeforeEach
				void mock() {
					when(userManagerUrlProvider.isConfiguredForUserProfile()).thenReturn(true);
				}

				@Test
				void shouldBePresent() {
					when(userManagerUrlProvider.getUserProfileTemplate()).thenReturn(userProfileTemplate);

					var link = modelAssembler.toModel(PostfachMailTestFactory.create()).getLink(PostfachMailModelAssembler.REL_CREATED_BY);

					assertThat(link).isPresent().get().extracting(Link::getHref)
							.isEqualTo("UserProfileTemplateDummy/" + PostfachMailTestFactory.CREATED_BY);
				}

				@DisplayName("should not be present if the value of createdBy starts with 'system'")
				@Test
				void shouldNOTBePresentOnSystemUser() {
					var link = modelAssembler
							.toModel(PostfachMailTestFactory.createBuilder()
									.createdBy(UserId.from(PostfachMailModelAssembler.SYSTEM_USER_IDENTIFIER + UUID.randomUUID().toString())).build())
							.getLink(PostfachMailModelAssembler.REL_CREATED_BY);

					assertThat(link).isNotPresent();
				}

				@Test
				void shouldNotBePresentOnNull() {
					var link = modelAssembler.toModel(PostfachMailTestFactory.createBuilder().createdBy(null).build())
							.getLink(PostfachMailModelAssembler.REL_CREATED_BY);

					assertThat(link).isNotPresent();
				}
			}

			@Nested
			class TestOnNonConfiguredUserManager {

				@BeforeEach
				void mock() {
					when(userManagerUrlProvider.isConfiguredForUserProfile()).thenReturn(false);
				}

				@Test
				void shouldNOTBePresentOnNotConfiguredUserManager() {
					var link = modelAssembler
							.toModel(PostfachMailTestFactory.createBuilder()
									.createdBy(UserId.from(PostfachMailModelAssembler.SYSTEM_USER_IDENTIFIER + UUID.randomUUID().toString())).build())
							.getLink(PostfachMailModelAssembler.REL_CREATED_BY);

					assertThat(link).isNotPresent();
				}

				@Test
				void shouldNotCallUserManagerUrlProvider() {
					modelAssembler
							.toModel(PostfachMailTestFactory.createBuilder()
									.createdBy(UserId.from(PostfachMailModelAssembler.SYSTEM_USER_IDENTIFIER + UUID.randomUUID().toString())).build())
							.getLink(PostfachMailModelAssembler.REL_CREATED_BY);

					verify(userManagerUrlProvider, never()).getUserProfileTemplate();
				}

				@Test
				void shouldNotBePresentOnNull() {
					var link = modelAssembler.toModel(PostfachMailTestFactory.createBuilder().createdBy(null).build())
							.getLink(PostfachMailModelAssembler.REL_CREATED_BY);

					assertThat(link).isNotPresent();
				}
			}
		}

		@DisplayName("edit link")
		@Nested
		class TestEditLink {
			PostfachMail.PostfachMailBuilder UNSENT_MAIL_BUILDER = PostfachMailTestFactory.createBuilder()
					.direction(Direction.OUT).sentAt(null).sentSuccessful(false);

			@Test
			void shouldBePresentOnUnsendNachricht() {
				var postfachNachricht = UNSENT_MAIL_BUILDER.build();

				var link = modelAssembler.toModel(postfachNachricht).getLink(PostfachMailModelAssembler.REL_EDIT);

				assertThat(link).isPresent().get().extracting(Link::getHref)
						.isEqualTo("/api/postfachMails/" + PostfachMailTestFactory.ID + "/commands");
			}

			@Nested
			class ShouldNotBePresent {

				@Test
				void OnIncomingNachricht() {
					var postfachNachricht = UNSENT_MAIL_BUILDER.direction(Direction.IN).build();

					var link = modelAssembler.toModel(postfachNachricht).getLink(PostfachMailModelAssembler.REL_EDIT);

					assertThat(link).isNotPresent();
				}

				@Test
				void OnSentNachricht() {
					var postfachNachricht = UNSENT_MAIL_BUILDER.sentSuccessful(true).build();

					var link = modelAssembler.toModel(postfachNachricht).getLink(PostfachMailModelAssembler.REL_EDIT);

					assertThat(link).isNotPresent();
				}
			}
		}
	}

	@DisplayName("On collection model link")
	@Nested
	class TestLinksOnCollectionModel {

		@Test
		void shouldHaveSelfLink() {
			var model = modelAssembler
					.toCollectionModel(Stream.of(PostfachMailTestFactory.create()), PostfachTestFactory.create(),
							VorgangWithEingangTestFactory.create());
			var selfLink = model.getLink(IanaLinkRelations.SELF_VALUE);

			assertThat(selfLink).isPresent();
			assertThat(selfLink.get().getHref()).isEqualTo(
					String.format("%s?%s=%s", PostfachMailController.PATH, PostfachMailController.PARAM_VORGANG_ID, VorgangHeaderTestFactory.ID));
		}

		@DisplayName("send postfach mail")
		@Nested
		class SendPostfachMailLink {

			private final String sendPostfachMailLinkRel = PostfachMailModelAssembler.REL_SEND_POSTFACH_MAIL;

			@Test
			void shouldExistsOnExistingServiceKonto() {
				var link = toCollectionModel(VorgangWithEingangTestFactory.create()).getLink(sendPostfachMailLinkRel);

				assertThat(link).isPresent();
				assertThat(link.get().getHref())
						.isEqualTo(PostfachMailCommandByVorgangController.PATH.replace("{vorgangId}",
								VorgangHeaderTestFactory.ID));
			}

			@Test
			void shouldNotExist() {
				var link = toCollectionModel(VorgangWithEingangTestFactory.createBuilder().status(VorgangStatus.ZU_LOESCHEN).build()).getLink(
						sendPostfachMailLinkRel);

				assertThat(link).isEmpty();
			}

			@DisplayName("should NOT exists")
			@Nested
			class TestNotExists {

				@Test
				void onMissingVorgangHeader() {
					var vorgangWithoutHeader = VorgangWithEingangTestFactory.createBuilder().header(null).build();

					var link = toCollectionModel(vorgangWithoutHeader).getLink(sendPostfachMailLinkRel);

					assertThat(link).isNotPresent();
				}

				@Test
				void onMissingServiceKonto() {
					var vorgangWithoutServiceKonto = VorgangWithEingangTestFactory.createBuilder().header(
							VorgangHeadTestFactory.createBuilder().serviceKonto(null).build()).build();

					var link = toCollectionModel(vorgangWithoutServiceKonto).getLink(sendPostfachMailLinkRel);

					assertThat(link).isNotPresent();
				}
			}
		}

		@Nested
		class UploadAttachmentLink {

			private final String uploadAttachmentLinkRel = PostfachMailModelAssembler.REL_UPLOAD_ATTACHMENT;

			@Test
			void shouldHaveLink() {
				var link = toCollectionModel(VorgangWithEingangTestFactory.create()).getLink(uploadAttachmentLinkRel);

				assertThat(link).isPresent();
				assertThat(link.get().getHref())
						.isEqualTo(String.format("/api/binaryFiles/%s/postfachNachrichtAttachment/file", VorgangHeaderTestFactory.ID));
			}

			@Test
			void shouldNotHaveLink() {
				var vorgangWithoutServiceKonto = VorgangWithEingangTestFactory.createBuilder().header(
						VorgangHeadTestFactory.createBuilder().serviceKonto(null).build()).build();
				var link = toCollectionModel(vorgangWithoutServiceKonto).getLink(uploadAttachmentLinkRel);

				assertThat(link).isNotPresent();
			}
		}

		private RepresentationModel<EntityModel<Postfach>> toCollectionModel(VorgangWithEingang vorgang) {
			return modelAssembler.toCollectionModel(
					Stream.of(PostfachMailTestFactory.create()),
					buildPostfach(),
					vorgang);
		}

		private Postfach buildPostfach() {
			return PostfachTestFactory.createBuilder()
					.features(FeaturesTestFactory.createBuilder()
							.reply(FeaturesTestFactory.REPLY_FORBIDDEN)
							.build())
					.build();
		}
	}
}