/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.time.LocalDate;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.common.errorhandling.TechnicalException;

class WiedervorlageCommandControllerTest {

	@Spy
	@InjectMocks
	private WiedervorlageCommandController controller;
	@Mock
	private CommandService commandService;
	@Mock
	private WiedervorlageService service;
	@Captor
	private ArgumentCaptor<WiedervorlageCommand> commandCaptor;
	@Captor
	private ArgumentCaptor<CreateCommand> createCommandCaptor;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@DisplayName("Update wiedervorlage")
	@Nested
	class TestUpdateWiedervorlage {

		private static final String RESPONSE_HEADER = "http://localhost/api/commands/" + WiedervorlageCommandTestFactory.ID;

		@Nested
		class ControllerMethods {

			@BeforeEach
			void init() {
				when(service.getById(any())).thenReturn(WiedervorlageTestFactory.create());
				when(service.editWiedervorlage(any(), any(), anyLong())).thenReturn(CommandTestFactory.createBuilder()
						.order(CommandOrder.UPDATE_ATTACHED_ITEM)
						.body(WiedervorlageTestFactory.createAsMap()).build());
			}

			@Test
			void shouldCallServiceGetById() throws Exception {
				doRequest();

				verify(service).getById(WiedervorlageTestFactory.ID);
			}

			@Test
			void shouldCallServiceUpdateNextFrist() throws Exception {
				doRequest();

				verify(service).updateNextFrist(VorgangHeaderTestFactory.ID);
			}

			@Test
			void shouldReturnCreatedStatus() throws Exception {
				doRequest().andExpect(status().isCreated());
			}

			@Test
			void shouldResponseWithLocationHeader() throws Exception {
				doRequest().andExpect(header().string("Location", RESPONSE_HEADER));
			}

			@Test
			void shouldCreateCommand() throws Exception {
				doRequest();

				verify(controller).createCommand(any(Wiedervorlage.class), any(WiedervorlageCommand.class));
			}

			private ResultActions doRequest() throws Exception {
				return mockMvc.perform(
						post(WiedervorlageCommandController.WIEDERVORLAGE_COMMANDS, WiedervorlageTestFactory.ID, WiedervorlageTestFactory.VERSION)
								.content(WiedervorlageCommandTestFactory.createValidRequestContent()).contentType(MediaType.APPLICATION_JSON))
						.andExpect(status().is2xxSuccessful());
			}
		}

		@DisplayName("create command")
		@Nested
		class TestCreateCommand {

			@DisplayName("for order 'erledigen'")
			@Nested
			class TestOnErledigenOrder {

				@Test
				void shouldCallService() {
					callCreateCommand(CommandOrder.WIEDERVORLAGE_ERLEDIGEN);

					verify(service).erledigen(any(Wiedervorlage.class));
				}
			}

			@DisplayName("for order 'wiedereroeffnen'")
			@Nested
			class TestOnWiedereroeffnenOrder {

				@Test
				void shouldCallService() {
					callCreateCommand(CommandOrder.WIEDERVORLAGE_WIEDEREROEFFNEN);

					verify(service).wiedereroeffnen(any(Wiedervorlage.class));
				}
			}

			@DisplayName("for order 'edit'")
			@Nested
			class TestOnEditOrder {

				@Test
				void shouldCallService() {
					callCreateCommand(CommandOrder.EDIT_WIEDERVORLAGE);

					verify(service).editWiedervorlage(any(Wiedervorlage.class), eq(WiedervorlageTestFactory.ID),
							eq(WiedervorlageTestFactory.VERSION));
				}

				@DisplayName("update wiedervorlage by given command")
				@Nested
				class TestUpdateWiedervorlageByCommand {

					private final FileId ATTACHMENT_ID = FileId.from("73");
					private final String BETREFF = "Neuer Betreff";
					private final String BESCHREIBUNG = "Neuer Beschreibung";
					private final LocalDate FRIST = LocalDate.now();

					@Test
					void shouldHaveSetAttachments() {
						var updatedWiedervorlage = update();

						assertThat(updatedWiedervorlage.getAttachments()).hasSize(1);
						assertThat(updatedWiedervorlage.getAttachments().get(0)).isEqualTo(ATTACHMENT_ID);
					}

					@Test
					void shouldHaveSetBetreff() {
						var updatedWiedervorlage = update();

						assertThat(updatedWiedervorlage.getBetreff()).isEqualTo(BETREFF);
					}

					@Test
					void shouldHaveSetBeschreibung() {
						var updatedWiedervorlage = update();

						assertThat(updatedWiedervorlage.getBeschreibung()).isEqualTo(BESCHREIBUNG);
					}

					@Test
					void shouldHaveSetFrist() {
						var updatedWiedervorlage = update();

						assertThat(updatedWiedervorlage.getFrist()).isEqualTo(FRIST);
					}

					private Wiedervorlage update() {
						return controller.updateWiedervorlageByCommand(WiedervorlageTestFactory.create(), buildWiedervorlageCommand());
					}

					private WiedervorlageCommand buildWiedervorlageCommand() {
						var commandWiedervorlage = WiedervorlageTestFactory.createBuilder()
								.clearAttachments().attachment(ATTACHMENT_ID)
								.betreff(BETREFF)
								.beschreibung(BESCHREIBUNG)
								.frist(FRIST)
								.build();
						return WiedervorlageCommandTestFactory.createBuilder().wiedervorlage(commandWiedervorlage).build();
					}
				}
			}

			@Nested
			class OnUnsupportedOrder {

				@Test
				void shouldThrowException() {
					var wiedervorlage = WiedervorlageTestFactory.create();
					var command = WiedervorlageCommandTestFactory.createBuilder().order(CommandOrder.CREATE_ATTACHED_ITEM).build();

					assertThatExceptionOfType(TechnicalException.class)
							.isThrownBy(() -> controller.createCommand(wiedervorlage, command));
				}
			}

			private Command callCreateCommand(CommandOrder order) {
				return controller.createCommand(WiedervorlageTestFactory.create(),
						WiedervorlageCommandTestFactory.createBuilder().order(order).build());
			}
		}
	}
}