/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static de.ozgcloud.alfa.wiedervorlage.WiedervorlageCommandTestFactory.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.time.LocalDate;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.ValidationMessageCodes;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandRemoteService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.wiedervorlage.WiedervorlageCommandController.WiedervorlageCommandByVorgangController;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
class WiedervorlageCommandITCase {

	@Autowired
	private MockMvc mockMvc;
	@MockBean
	private CommandRemoteService commandRemoteService;
	@MockBean
	private WiedervorlageRemoteService wiedervorlageRemoteService;

	@SpyBean
	private CurrentUserService userService;

	@DisplayName("Create edit command")
	@WithMockUser
	@Nested
	class TestCreateEditCommand {

		@BeforeEach
		void initTest() {
			when(wiedervorlageRemoteService.getById(anyString())).thenReturn(WiedervorlageTestFactory.create());
			when(commandRemoteService.createCommand(any())).thenReturn(CommandTestFactory.create());
		}

		@Test
		void shouldCreateCommand() throws Exception {
			doRequest(createValidRequestContent()).andExpect(status().isCreated());

			verify(commandRemoteService).createCommand(any());
		}

		@WithMockUser
		@DisplayName("should return validation error")
		@Nested
		class TestValidation {

			@DisplayName("for null Betreff")
			@Test
			void createCommandWithInvalidBetreff() throws Exception {
				String content = buildContentWithBetreff(null);

				doRequest(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].field").value("wiedervorlage.betreff"))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
			}

			@DisplayName("for only 1 character in Betreff")
			@Test
			void createcommandWithShortBetreff() throws Exception {
				String content = buildContentWithBetreff("a");

				doRequest(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.[0].field").value("wiedervorlage.betreff"));

			}

			@DisplayName("for invalid betreff should have paramter")
			@Test
			void minMaxParameter() throws Exception {
				String content = buildContentWithBetreff("a");

				doRequest(content).andExpect(status().isUnprocessableEntity()).andExpect(jsonPath("$.issues[0].parameters.length()").value(2));
			}

			@DisplayName("for 41 character in Betreff")
			@Test
			void createCommandWithLongBetreff() throws Exception {
				var content = buildContentWithBetreff(RandomStringUtils.random(41));

				doRequest(content).andExpect(status().isUnprocessableEntity());
			}

			private String buildContentWithBetreff(String betreff) {
				return createEditContent(WiedervorlageTestFactory.createBuilder().betreff(betreff).build());
			}

			@DisplayName("for frist in past")
			@Test
			void createCommandWithInvalidDate() throws Exception {
				String content = createEditContent(WiedervorlageTestFactory.createBuilder().frist(LocalDate.parse("2020-01-01")).build());

				doRequest(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].field").value("wiedervorlage.frist"))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_DATE_PAST));
			}

			@DisplayName("for empty frist")
			@Test
			void createCommandWithoutDate() throws Exception {
				String content = createEditContent(WiedervorlageTestFactory.createBuilder().frist(null).build());

				doRequest(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].field").value("wiedervorlage.frist"))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
			}

			private String createEditContent(Wiedervorlage wiedervorlage) {
				return createRequestContent(createWithWiedervorlage(wiedervorlage).toBuilder().order(CommandOrder.EDIT_WIEDERVORLAGE).build());
			}

		}

		private ResultActions doRequest(String content) throws Exception {
			return mockMvc.perform(
					post(WiedervorlageCommandController.WIEDERVORLAGE_COMMANDS, WiedervorlageTestFactory.ID, WiedervorlageTestFactory.VERSION)
							.with(csrf())
							.contentType(MediaType.APPLICATION_JSON)
							.content(content));
		}
	}

	@DisplayName("Create create command")
	@WithMockUser
	@Nested
	class TestCreateCreateCommand {

		private static final long RELATION_ID_ON_CREATE = -1;

		@BeforeEach
		void initTest() {
			when(commandRemoteService.createCommand(any())).thenReturn(CommandTestFactory.create());
			when(wiedervorlageRemoteService.getById(anyString())).thenReturn(WiedervorlageTestFactory.create());

			doReturn(UserProfileTestFactory.ID).when(userService).getUserId();
		}

		@Test
		void shouldCreateCommand() throws Exception {
			doRequest(createValidRequestContent()).andExpect(status().isCreated());

			verify(commandRemoteService).createCommand(any());
		}

		@WithMockUser
		@DisplayName("should return validation error")
		@Nested
		class TestValidation {

			@DisplayName("for null Betreff")
			@Test
			void createCommandWithInvalidBetreff() throws Exception {
				String content = buildContentWithBetreff(null);

				doRequest(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].field").value("wiedervorlage.betreff"))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
			}

			@DisplayName("for only 1 character in Betreff")
			@Test
			void createcommandWithShortBetreff() throws Exception {
				String content = buildContentWithBetreff("a");

				doRequest(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.[0].field").value("wiedervorlage.betreff"));

			}

			@DisplayName("for invalid betreff should have paramter")
			@Test
			void minMaxParameter() throws Exception {
				String content = buildContentWithBetreff("a");

				doRequest(content).andExpect(status().isUnprocessableEntity()).andExpect(jsonPath("$.issues[0].parameters.length()").value(2));
			}

			@DisplayName("for 41 character in Betreff")
			@Test
			void createCommandWithLongBetreff() throws Exception {
				var content = buildContentWithBetreff(RandomStringUtils.randomAlphanumeric(41));

				doRequest(content).andExpect(status().isUnprocessableEntity());
			}

			private String buildContentWithBetreff(String betreff) {
				return createRequestContent(createWithWiedervorlage(WiedervorlageTestFactory.createBuilder().betreff(betreff).build()));
			}

			@DisplayName("for frist in past")
			@Test
			void createCommandWithInvalidDate() throws Exception {
				String content = createRequestContent(
						createWithWiedervorlage(WiedervorlageTestFactory.createBuilder().frist(LocalDate.parse("2020-01-01")).build()));

				doRequest(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].field").value("wiedervorlage.frist"))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_DATE_PAST));
			}

			@DisplayName("for empty frist")
			@Test
			void createCommandWithoutDate() throws Exception {
				String content = createRequestContent(
						createWithWiedervorlage(WiedervorlageTestFactory.createBuilder().frist(null).build()));

				doRequest(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].field").value("wiedervorlage.frist"))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
			}

		}

		private ResultActions doRequest(String content) throws Exception {
			return mockMvc.perform(post(WiedervorlageCommandByVorgangController.WIEDERVORLAGE_COMMANDS_BY_VORGANG, VorgangHeaderTestFactory.ID,
					RELATION_ID_ON_CREATE)
					.with(csrf())
					.contentType(MediaType.APPLICATION_JSON)
					.content(content));
		}
	}
}