package de.ozgcloud.alfa.export;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.ExportFilenameGenerator;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.file.ExportFileService;
import de.ozgcloud.alfa.vorgang.ExportVorgangService;
import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.xoev.xdomea.AbgabeAbgabe0401;

@Service
class ExportService {

	static final String EXPORT_FILENAME_SUFFIX = "_Abgabe.Abgabe.0401.xml";
	private static final String EXPORT_FILENAME_TEMPLATE = "%s" + EXPORT_FILENAME_SUFFIX;

	@Autowired
	private XDomeaXmlMarshaller xDomeaXmlMarshaller;

	@Autowired
	private ExportFileService exportFileService;

	@Autowired
	private ExportVorgangService exportVorgangService;

	@Autowired
	private ExportFilenameGenerator exportFilenameGenerator;

	public void writeExport(String vorgangId, String filenameId, OutputStream out) {
		var exportData = collectExportData(vorgangId, filenameId);
		var zipFile = createZipFile(exportData);
		writeZipFileContent(zipFile, out);
	}

	ExportData collectExportData(String vorgangId, String filenameId) {
		var vorgangWithEingang = exportVorgangService.getVorgang(vorgangId);
		var ozgFiles = exportFileService.getAllPdfs(vorgangWithEingang.getEingang().getId());
		var abgabe = AbgabeCreator.createAbgabeCreator()
				.withKopf(exportVorgangService.createKopf(vorgangWithEingang))
				.withVorgang(exportVorgangService.createVorgangType(vorgangWithEingang))
				.withDokumentTypes(
						exportFileService.createDokumentTypes(ozgFiles.stream(), vorgangWithEingang.getEingang().getHeader().getFormEngineName()))
				.create();
		return ExportData.builder().abgabe(abgabe).exportFilename(buildXmlFilename(filenameId)).exportFiles(ozgFiles.stream()).build();
	}

	String buildXmlFilename(String filenameId) {
		return String.format(EXPORT_FILENAME_TEMPLATE, filenameId);
	}

	String createXmlContent(AbgabeAbgabe0401 abgabe) {
		return xDomeaXmlMarshaller.marshal(abgabe);
	}

	File createZipFile(ExportData exportData) {
		var file = TempFileUtils.createTmpFile().toFile();
		try (var zipOutputStream = new ZipOutputStream(new FileOutputStream(file))) {
			putZipEntry(exportData.getExportFilename(), createXmlContent(exportData.getAbgabe()), zipOutputStream);
			putFilesIntoZip(exportData.getExportFiles(), zipOutputStream);
			return file;
		} catch (Exception e) {
			throw new TechnicalException("Error creating XDomea zip file", e);
		}
	}

	void putZipEntry(String fileName, String fileData, ZipOutputStream zipOutputStream) throws IOException {
		var entry = new ZipEntry(fileName);
		zipOutputStream.putNextEntry(entry);
		zipOutputStream.write(fileData.getBytes(StandardCharsets.UTF_8));
		zipOutputStream.closeEntry();
	}

	private void putFilesIntoZip(Stream<OzgFile> ozgFiles, ZipOutputStream zipOutputStream) {
		ozgFiles.forEach(ozgFile -> putOzgFileIntoZip(ozgFile, zipOutputStream));
	}

	void putOzgFileIntoZip(OzgFile ozgFile, ZipOutputStream zipOutputStream) {
		try {
			var entry = new ZipEntry(exportFilenameGenerator.generateExportFilename(ozgFile));
			zipOutputStream.putNextEntry(entry);
			exportFileService.writeOzgFile(ozgFile.getId(), zipOutputStream);
			zipOutputStream.closeEntry();
		} catch (IOException e) {
			throw new TechnicalException("Cannot add file to ZIP.", e);
		}
	}

	void writeZipFileContent(File file, OutputStream outputStream) {
		try (var fileInputStream = new FileInputStream(file)) {
			fileInputStream.transferTo(outputStream);
		} catch (Exception e) {
			throw new TechnicalException("Error writing XDomea zip file to output stream", e);
		}
	}
}
