package de.ozgcloud.alfa.file;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ExportFilenameGenerator;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.FormatType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.MediumCodeType;
import de.xoev.xdomea.PrimaerdokumentType;
import de.xoev.xdomea.VersionType;

@Component
class DokumentTypeCreator {

	static final String VERSION_NUMMER = "1";
	static final String ALLGEMEINE_METADATEN_MEDIUM_CODE = "001";

	@Autowired
	private UUIDConverter uuidConverter;

	@Autowired
	private ExportFilenameGenerator exportFilenameGenerator;

	public Stream<DokumentType> create(Stream<OzgFile> ozgFiles, String formEngineName) {
		final var ordinalNumberGenerator = new AtomicInteger(1);
		return ozgFiles.map(ozgFile -> createDokumentType(new MappingData(ozgFile, ordinalNumberGenerator.getAndIncrement(), formEngineName)));
	}

	DokumentType createDokumentType(MappingData mappingData) {
		var dokumentType = new DokumentType();
		dokumentType.setIdentifikation(createIdentifikation(mappingData));
		dokumentType.setAllgemeineMetadaten(createAllgemeineMetadaten(mappingData));

		var versionType = createVersionType();
		dokumentType.getVersion().add(versionType);

		var formatType = createFormatType();
		versionType.getFormat().add(formatType);

		var primaerdokumentType = createPrimaerdokumentType(mappingData);
		formatType.setPrimaerdokument(primaerdokumentType);

		return dokumentType;
	}

	IdentifikationObjektType createIdentifikation(MappingData mappingData) {
		var identifikation = new IdentifikationObjektType();
		identifikation.setID(uuidConverter.fromObjectId(mappingData.ozgFile().getId().toString()));
		identifikation.setNummerImUebergeordnetenContainer(Long.valueOf(mappingData.dokumentOrdinalNumber()));
		return identifikation;
	}

	AllgemeineMetadatenType createAllgemeineMetadaten(MappingData mappingData) {
		var allgemeineMetadaten = new AllgemeineMetadatenType();
		allgemeineMetadaten.setBetreff(mappingData.ozgFile().getName());
		allgemeineMetadaten.setKennzeichen(StringUtils.EMPTY);
		allgemeineMetadaten.setBemerkung(StringUtils.EMPTY);
		allgemeineMetadaten.setMedium(createMediumCode());
		return allgemeineMetadaten;
	}

	private MediumCodeType createMediumCode() {
		var mediumCode = new MediumCodeType();
		mediumCode.setCode(ALLGEMEINE_METADATEN_MEDIUM_CODE);
		return mediumCode;
	}

	private VersionType createVersionType() {
		var versionType = new VersionType();
		versionType.setNummer(VERSION_NUMMER);
		return versionType;
	}

	private FormatType createFormatType() {
		var formatType = new FormatType();
		formatType.setName(DateiformatCode.PDF.createDateiformatCodeType());
		formatType.setSonstigerName(StringUtils.EMPTY);
		formatType.setVersion(StringUtils.EMPTY);
		return formatType;
	}

	private PrimaerdokumentType createPrimaerdokumentType(MappingData mappingData) {
		var primaerdokumentType = new PrimaerdokumentType();
		primaerdokumentType.setDateiname(exportFilenameGenerator.generateExportFilename(mappingData.ozgFile()));
		primaerdokumentType.setDateinameOriginal(mappingData.ozgFile().getName());
		primaerdokumentType.setErsteller(mappingData.formEngineName());
		return primaerdokumentType;
	}

	record MappingData (OzgFile ozgFile, int dokumentOrdinalNumber, String formEngineName) {}
}
