package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.assertThatThrownBy;
import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.UUID;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.ExportFilenameGenerator;
import de.ozgcloud.alfa.common.TestUtils;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.file.ExportFileService;
import de.ozgcloud.alfa.vorgang.EingangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.EingangTestFactory;
import de.ozgcloud.alfa.vorgang.ExportVorgangService;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.xoev.xdomea.AbgabeAbgabe0401;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.VorgangType;
import lombok.SneakyThrows;

class ExportServiceTest {

	@Spy
	@InjectMocks
	private ExportService service;

	@Mock
	private XDomeaXmlMarshaller XDomeaXmlMarshaller;

	@Mock
	private ExportFileService exportFileService;

	@Mock
	private ExportVorgangService exportVorgangService;

	@Mock
	private ExportFilenameGenerator exportFilenameGenerator;

	@DisplayName("Write exportToXdomea")
	@Nested
	class TestWriteExport {

		private static final String FILENAME_ID = UUID.randomUUID().toString();
		private final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		private final ExportData exportData = ExportDataTestFactory.create();

		@Mock
		private File zipFile;

		@BeforeEach
		void setUp() {
			doReturn(exportData).when(service).collectExportData(VorgangHeaderTestFactory.ID, FILENAME_ID);
			doReturn(zipFile).when(service).createZipFile(exportData);
			doNothing().when(service).writeZipFileContent(zipFile, outputStream);
		}

		@Test
		void shouldCollectExportData() {
			callService();

			verify(service).collectExportData(VorgangHeaderTestFactory.ID, FILENAME_ID);
		}

		@Test
		void shouldCreateZipFile() {
			callService();

			verify(service).createZipFile(exportData);
		}

		@Test
		void shouldWriteZipFileContentToOutputStream() {
			callService();

			verify(service).writeZipFileContent(zipFile, outputStream);
		}

		private void callService() {
			service.writeExport(VorgangHeaderTestFactory.ID, FILENAME_ID, outputStream);
		}
	}

	@Nested
	class TestCollectExportData {

		private static final String FILENAME_ID = UUID.randomUUID().toString();
		private static final String FILE_NAME = "file.zip";
		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final VorgangType vorgangType = new VorgangType();
		private final NkAbgabeType kopfType = new NkAbgabeType();
		private final Stream<OzgFile> ozgFiles = Stream.of(OzgFileTestFactory.create());
		private final Stream<DokumentType> dokumentTypes = Stream.of(DokumentTypeTestFactory.create());
		private final AbgabeAbgabe0401 abgabeAbgabe0401 = AbgabeAbgabe0401TestFactory.create();

		private MockedStatic<AbgabeCreator> abgabeCreatorStatic;

		@Spy
		private AbgabeCreator abgabeCreator;

		@BeforeEach
		void setUp() {
			when(exportVorgangService.getVorgang(any())).thenReturn(vorgang);
			when(exportVorgangService.createVorgangType(any())).thenReturn(vorgangType);
			when(exportVorgangService.createKopf(any())).thenReturn(kopfType);
			List<OzgFile> ozgFileList = mock(List.class);
			when(ozgFileList.stream()).thenReturn(ozgFiles);
			when(exportFileService.getAllPdfs(any())).thenReturn(ozgFileList);
			when(exportFileService.createDokumentTypes(any(), any())).thenReturn(dokumentTypes);
			doReturn(FILE_NAME).when(service).buildXmlFilename(any());
			abgabeCreatorStatic = mockStatic(AbgabeCreator.class);
			abgabeCreatorStatic.when(AbgabeCreator::createAbgabeCreator).thenReturn(abgabeCreator);
			doReturn(abgabeAbgabe0401).when(abgabeCreator).create();
		}

		@AfterEach
		void cleanup() {
			abgabeCreatorStatic.close();
		}

		@Test
		void shouldLoadVorgang() {
			callService();

			verify(exportVorgangService).getVorgang(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldGenerateXmlFilename() {
			callService();

			verify(service).buildXmlFilename(FILENAME_ID);
		}

		@Test
		void shouldGetPdfFiles() {
			callService();

			verify(exportFileService).getAllPdfs(EingangTestFactory.ID);
		}

		@Test
		void shouldCreateKopf() {
			callService();

			verify(exportVorgangService).createKopf(vorgang);
		}

		@Test
		void shouldCreateVorgangType() {
			callService();

			verify(exportVorgangService).createVorgangType(vorgang);
		}

		@Test
		void shouldCreateDokumentTypes() {
			callService();

			verify(exportFileService).createDokumentTypes(ozgFiles, EingangHeaderTestFactory.FORM_ENGINE_NAME);
		}

		@Test
		void shouldSetKopf() {
			callService();

			verify(abgabeCreator).withKopf(kopfType);
		}

		@Test
		void shouldSetVorgang() {
			callService();

			verify(abgabeCreator).withVorgang(vorgangType);
		}

		@Test
		void shouldSetDokumentTypes() {
			callService();

			verify(abgabeCreator).withDokumentTypes(dokumentTypes);
		}

		@Test
		void shouldCreateAbgabe() {
			callService();

			verify(abgabeCreator).create();
		}

		@Test
		void shouldContainAbgabe() {
			var exportData = callService();

			assertThat(exportData.getAbgabe()).isEqualTo(abgabeAbgabe0401);
		}

		@Test
		void shouldContainExportFilename() {
			var exportData = callService();

			assertThat(exportData.getExportFilename()).isEqualTo(FILE_NAME);
		}

		@Test
		void shouldContainExportExportFiles() {
			var exportData = callService();

			assertThat(exportData.getExportFiles()).isEqualTo(ozgFiles);
		}

		private ExportData callService() {
			return service.collectExportData(VorgangHeaderTestFactory.ID, FILENAME_ID);
		}

	}

	@DisplayName("Build xml Filename")
	@Nested
	class TestBuildXmlFilename {

		@Test
		void shouldMatchPattern() {
			var filename = service.buildXmlFilename(UUID.randomUUID().toString());

			assertThat(filename).matches(TestUtils.uuidRegexWithSuffix(ExportService.EXPORT_FILENAME_SUFFIX));
		}
	}

	@DisplayName("Create xml content")
	@Nested
	class TestCreateXmlContent {

		private static final String XML_STRING = "<xml>";

		@BeforeEach
		void init() {
			when(XDomeaXmlMarshaller.marshal(any())).thenReturn(XML_STRING);
		}

		@Test
		void shouldCallXmlMarshaller() {
			callService();

			verify(XDomeaXmlMarshaller).marshal(ExportDataTestFactory.ABGABE);
		}

		@Test
		void shouldReturnMarshalledString() {
			var result = callService();

			assertThat(result).isEqualTo(XML_STRING);
		}

		private String callService() {
			return service.createXmlContent(ExportDataTestFactory.ABGABE);
		}
	}

	@Nested
	class TestCreateZipFile {

		private static final String XML_FILE_CONTENT = "<xml></xml>";
		private static final String XML_STRING = "<xml>";
		private final ExportData exportData = ExportDataTestFactory.create();

		@Captor
		private ArgumentCaptor<ZipOutputStream> zipOutputStreamArgumentCaptor;

		@SneakyThrows
		@BeforeEach
		void setUp() {
			when(service.createXmlContent(ExportDataTestFactory.ABGABE)).thenReturn(XML_FILE_CONTENT);
			doNothing().when(service).putOzgFileIntoZip(any(OzgFile.class), any(ZipOutputStream.class));
		}

		@Test
		void shouldCreateXmlStringContent() {
			callService();

			verify(service).createXmlContent(ExportDataTestFactory.ABGABE);
		}

		@Test
		void shouldCreateZipEntry() throws IOException {
			callService();

			verify(service).putZipEntry(eq(ExportDataTestFactory.EXPORT_FILENAME), eq(XML_FILE_CONTENT),
					any(ZipOutputStream.class));
		}

		@Test
		void shouldCreateZipOutputStream() throws IOException {
			callService();

			verify(service).putZipEntry(eq(ExportDataTestFactory.EXPORT_FILENAME), eq(XML_FILE_CONTENT),
					zipOutputStreamArgumentCaptor.capture());
			assertThat(zipOutputStreamArgumentCaptor.getValue()).isInstanceOf(ZipOutputStream.class);
		}

		@Test
		void shouldWriteBytes() {
			var file = callService();

			assertThat(file).isNotEmpty().content().hasSizeGreaterThan(100);
		}

		@SneakyThrows
		@Test
		void shouldWritePdfFiles() {
			callService();

			verify(service).putOzgFileIntoZip(eq(ExportDataTestFactory.OZG_FILE), any(ZipOutputStream.class));
		}

		@Test
		void shouldThrowTechnicalException() throws IOException {
			doThrow(IOException.class).when(service).putZipEntry(anyString(), eq(XML_STRING), any(ZipOutputStream.class));

			assertThatThrownBy(this::callService).isInstanceOf(TechnicalException.class);
		}

		private File callService() {
			return service.createZipFile(exportData);
		}

	}

	@Nested
	class TestPutZipEntry {

		@Mock
		private ZipOutputStream zipOutputStream;

		@Captor
		private ArgumentCaptor<ZipEntry> zipEntryArgumentCaptor;

		@Captor
		private ArgumentCaptor<byte[]> fileDataArgumentCaptor;

		private final String FILE_NAME = "file.xml";
		private final String FILE_DATA = "xml content";

		@SneakyThrows
		@Test
		void shouldPutNextEntry() {
			callService();

			verify(zipOutputStream).putNextEntry(zipEntryArgumentCaptor.capture());
			assertThat(zipEntryArgumentCaptor.getValue().getName()).isEqualTo(FILE_NAME);
		}

		@SneakyThrows
		@Test
		void shouldWriteDataBytes() {
			callService();

			verify(zipOutputStream).write(fileDataArgumentCaptor.capture());
			assertThat(fileDataArgumentCaptor.getValue()).isEqualTo(FILE_DATA.getBytes(StandardCharsets.UTF_8));
		}

		@SneakyThrows
		@Test
		void shouldCloseEntry() {
			callService();

			verify(zipOutputStream).closeEntry();
		}

		@SneakyThrows
		private void callService() {
			service.putZipEntry(FILE_NAME, FILE_DATA, zipOutputStream);
		}
	}

	@Nested
	class TestPutOzgFileIntoZip {

		@Mock
		private ZipOutputStream zipOutputStream;

		@Captor
		private ArgumentCaptor<ZipEntry> zipEntryArgumentCaptor;

		private final OzgFile ozgFile = OzgFileTestFactory.create();

		@BeforeEach
		void setUp() {
			when(exportFilenameGenerator.generateExportFilename(ozgFile)).thenReturn(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldGenerateExportFilename() {
			callService();

			verify(exportFilenameGenerator).generateExportFilename(ozgFile);
		}

		@SneakyThrows
		@Test
		void shouldPutNextEntry() {
			callService();

			verify(zipOutputStream).putNextEntry(zipEntryArgumentCaptor.capture());
			assertThat(zipEntryArgumentCaptor.getValue().getName()).isEqualTo(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldCallExportFileService() {
			callService();

			verify(exportFileService).writeOzgFile(OzgFileTestFactory.ID, zipOutputStream);
		}

		@SneakyThrows
		@Test
		void shouldCloseEntry() {
			callService();

			verify(zipOutputStream).closeEntry();
		}

		@SneakyThrows
		private void callService() {
			service.putOzgFileIntoZip(ozgFile, zipOutputStream);
		}

	}
}