package de.ozgcloud.alfa.export;

import static org.assertj.core.api.AssertionsForClassTypes.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.TestUtils;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class ExportVorgangControllerTest {

	@Spy
	@InjectMocks
	private ExportVorgangController controller;

	@Mock
	private ExportService xDomeaService;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestBuildZipFilename {

		@Test
		void shouldMatchPattern() {
			var filename = controller.buildZipFilename(UUID.randomUUID().toString());

			assertThat(filename).matches(TestUtils.uuidRegexWithSuffix("_Abgabe.Abgabe.0401.xdomea"));
		}
	}

	@Nested
	class TestExportToXdomea {

		private static final String VORGANG_EXPORT_FILENAME = "00000000-0000-0000-0000-000000000000_Abgabe.Abgabe.0401.xml";

		@Captor
		private ArgumentCaptor<String> filenameIdArgumentCaptor;

		@BeforeEach
		void init() {
			doReturn(VORGANG_EXPORT_FILENAME).when(controller).buildZipFilename(anyString());
		}

		@Test
		void shouldHaveContentDispositonHeader() throws Exception {
			doRequest().andExpect(header().string(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + VORGANG_EXPORT_FILENAME));
		}

		@Test
		void shouldCallXDomeaService() throws Exception {
			doRequest();

			verify(xDomeaService).writeExport(eq(VorgangHeaderTestFactory.ID), filenameIdArgumentCaptor.capture(), any());
			assertThat(filenameIdArgumentCaptor.getValue()).matches(TestUtils.UUID_REGEX);
		}

		@Test
		void shouldUseUUIDAsFilenameId() throws Exception {
			doRequest();

			verify(controller).buildZipFilename(filenameIdArgumentCaptor.capture());

			assertThat(filenameIdArgumentCaptor.getValue()).matches(TestUtils.UUID_REGEX);

		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(asyncDispatch(
							mockMvc.perform(get(ExportVorgangController.PATH + "/" + VorgangHeaderTestFactory.ID)
									.header(HttpHeaders.ACCEPT, MediaType.APPLICATION_OCTET_STREAM_VALUE)).andReturn()))
					.andExpect(status().isOk());
		}
	}

}