package de.ozgcloud.alfa.file;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.ExportFilenameGenerator;
import de.ozgcloud.alfa.common.TestUtils;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.export.AllgemeineMetadatenTypeTestFactory;
import de.ozgcloud.alfa.export.IdentifikationObjektTypeTestFactory;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.FormatType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.PrimaerdokumentType;
import de.xoev.xdomea.VersionType;

class DokumentTypeCreatorTest {

	private static final int DOKUMENT_ORDINAL_NUMBER = 1;
	private static final String FORM_ENGINE_NAME = "DUMMY_NAME";

	private final OzgFile ozgFile = OzgFileTestFactory.create();
	private final DokumentTypeCreator.MappingData mappingData = new DokumentTypeCreator.MappingData(ozgFile, DOKUMENT_ORDINAL_NUMBER, FORM_ENGINE_NAME);

	@Spy
	@InjectMocks
	private DokumentTypeCreator creator;

	@Mock
	private UUIDConverter uuidConverter;

	@Mock
	private ExportFilenameGenerator exportFilenameGenerator;

	@Nested
	class TestCreate {

		private final OzgFile ozgFile1 = OzgFileTestFactory.create();
		private final OzgFile ozgFile2 = OzgFileTestFactory.create();
		private final List<OzgFile> ozgFiles = List.of(ozgFile1, ozgFile2);
		private final DokumentType dokumentType1 = new DokumentType();
		private final DokumentType dokumentType2 = new DokumentType();

		@BeforeEach
		void init() {
			doReturn(dokumentType1, dokumentType2).when(creator).createDokumentType(any());
		}

		@Test
		void shouldCallCreateDokumentType() {
			create().toList(); // call toList() to finalize stream

			verify(creator).createDokumentType(new DokumentTypeCreator.MappingData(ozgFile1, 1, FORM_ENGINE_NAME));
			verify(creator).createDokumentType(new DokumentTypeCreator.MappingData(ozgFile2, 2, FORM_ENGINE_NAME));
		}

		@Test
		void shouldReturnDocumentTypes() {
			var dokumentTypes = create().toList();

			assertThat(dokumentTypes.get(0)).isEqualTo(dokumentType1);
			assertThat(dokumentTypes.get(1)).isEqualTo(dokumentType2);
		}

		private Stream<DokumentType> create() {
			return creator.create(ozgFiles.stream(), FORM_ENGINE_NAME);
		}
	}

	@Nested
	class TestCreateDokumentType {

		private final IdentifikationObjektType identifikationObjekt = IdentifikationObjektTypeTestFactory.create();
		private final AllgemeineMetadatenType allgemeineMetadaten = AllgemeineMetadatenTypeTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(identifikationObjekt).when(creator).createIdentifikation(mappingData);
			doReturn(allgemeineMetadaten).when(creator).createAllgemeineMetadaten(mappingData);
		}

		@Test
		void shouldCreateIdentifikation() {
			create();

			verify(creator).createIdentifikation(mappingData);
		}

		@Test
		void shouldHaveIdentifikation() {
			var dokument = create();

			assertThat(dokument.getIdentifikation()).isEqualTo(identifikationObjekt);
		}

		@Test
		void shouldCreateAllgemeineMetadaten() {
			create();

			verify(creator).createAllgemeineMetadaten(mappingData);
		}

		@Test
		void shouldHaveAllgemeineMetadaten() {
			var dokument = create();

			assertThat(dokument.getAllgemeineMetadaten()).isEqualTo(allgemeineMetadaten);
		}

		@Test
		void shouldHaveVersion() {
			var dokumentType = create();

			assertThat(dokumentType.getVersion()).size().isEqualTo(1);
		}

		@Nested
		class TestVersion {

			@Test
			void shouldHaveNummer() {
				assertThat(getVersion().getNummer()).isEqualTo(DokumentTypeCreator.VERSION_NUMMER);
			}

			@Test
			void shouldHaveFormat() {
				assertThat(getVersion().getFormat()).size().isEqualTo(1);
			}
		}

		@Nested
		class TestFormat {

			@Test
			void shouldHaveName() {
				assertThat(getFormat().getName()).usingRecursiveComparison().isEqualTo(DateiformatCode.PDF.createDateiformatCodeType());
			}

			@Test
			void shouldHaveSonstigerName() {
				assertThat(getFormat().getSonstigerName()).isEmpty();
			}

			@Test
			void shouldHaveVersion() {
				assertThat(getFormat().getVersion()).isEmpty();
			}

			@Test
			void shouldHavePrimaerdokument() {
				assertThat(getPrimaerdokument()).isNotNull();
			}
		}

		@Nested
		class TestPrimaerdokument {

			@BeforeEach
			void init() {
				when(exportFilenameGenerator.generateExportFilename(ozgFile)).thenReturn(ozgFile.getName());
			}

			@Test
			void shouldGenerateExportFilename() {
				create();

				verify(exportFilenameGenerator).generateExportFilename(ozgFile);
			}

			@Test
			void shouldHaveDateiname() {
				assertThat(getPrimaerdokument().getDateiname()).isEqualTo(ozgFile.getName());
			}

			@Test
			void shouldHaveDateinameOriginal() {
				assertThat(getPrimaerdokument().getDateinameOriginal()).isEqualTo(ozgFile.getName());
			}

			@Test
			void shouldHaveErsteller() {
				assertThat(getPrimaerdokument().getErsteller()).isEqualTo(FORM_ENGINE_NAME);
			}

		}

		private VersionType getVersion() {
			return create().getVersion().get(0);
		}

		private FormatType getFormat() {
			return getVersion().getFormat().get(0);
		}

		private PrimaerdokumentType getPrimaerdokument() {
			return getFormat().getPrimaerdokument();
		}

		private DokumentType create() {
			return creator.createDokumentType(mappingData);
		}
	}

	@Nested
	class TestCreateIdentifikation {

		private final String FILE_UUID = "64a820d3-6285-172a-c028-0000000026d0";

		@BeforeEach
		void init() {
			when(uuidConverter.fromObjectId(OzgFileTestFactory.ID.toString())).thenReturn(FILE_UUID);
		}

		@Test
		void shouldConvertObjectId() {
			create();

			verify(uuidConverter).fromObjectId(OzgFileTestFactory.ID.toString());
		}

		@Test
		void shouldHaveId() {
			var identifikation = create();

			assertThat(identifikation.getID()).matches(TestUtils.UUID_REGEX).isEqualTo(FILE_UUID);
		}

		@Test
		void shouldHaveNummerImUebergeordnetenContainer() {
			var identifikation = create();

			assertThat(identifikation.getNummerImUebergeordnetenContainer()).isEqualTo(DOKUMENT_ORDINAL_NUMBER);
		}

		private IdentifikationObjektType create() {
			return creator.createIdentifikation(mappingData);
		}
	}

	@Nested
	class TestCreateAllgemeineMetadaten {

		@Test
		void shouldHaveBetreff() {
			var allgemeineMetadaten = creator.createAllgemeineMetadaten(mappingData);

			assertThat(allgemeineMetadaten.getBetreff()).isEqualTo(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldHaveKennzeichen() {
			var allgemeineMetadaten = creator.createAllgemeineMetadaten(mappingData);

			assertThat(allgemeineMetadaten.getKennzeichen()).isEmpty();
		}

		@Test
		void shouldHaveBemerkung() {
			var allgemeineMetadaten = creator.createAllgemeineMetadaten(mappingData);

			assertThat(allgemeineMetadaten.getBemerkung()).isEmpty();
		}

		@Test
		void shouldHaveMedium() {
			var allgemeineMetadaten = creator.createAllgemeineMetadaten(mappingData);

			assertThat(allgemeineMetadaten.getMedium().getCode()).isEqualTo(DokumentTypeCreator.ALLGEMEINE_METADATEN_MEDIUM_CODE);
		}

	}

}