package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZoneOffset;
import java.time.ZonedDateTime;

import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.export.XDomeaProperties;
import de.xoev.xdomea.BehoerdenkennungType;
import de.xoev.xdomea.Code;
import de.xoev.xdomea.KontaktType;
import de.xoev.xdomea.NachrichtentypCodeType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.OrganisationseinheitType;
import de.xoev.xdomea.SystemType;

class KopfCreatorTest {

	@Spy
	@InjectMocks
	private KopfCreator creator;

	@Mock
	private XDomeaProperties xDomeaProperties;
	@Mock
	private DateConverter dateConverter;

	@Nested
	class TestCreateKopf {

		static final ZonedDateTime CREATION_TIME = ZonedDateTime.now();

		@Test
		void shouldSetProcessId() {
			var kopf = createKopf();

			assertThat(kopf.getProzessID()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldSetNachrichtentyp() {
			var expectedValue = new NachrichtentypCodeType();
			doReturn(expectedValue).when(creator).createNachrichtentyp();

			var kopf = createKopf();

			assertThat(kopf.getNachrichtentyp()).isEqualTo(expectedValue);
		}

		@Test
		void shouldSetErstellungszeitpunkt() {
			try (var zonedDateTime = Mockito.mockStatic(ZonedDateTime.class)) {
				zonedDateTime.when(() -> ZonedDateTime.now(ZoneOffset.UTC)).thenReturn(CREATION_TIME);

				var expectedValue = mock(XMLGregorianCalendar.class);
				when(dateConverter.toXmlGregorianCalendar(CREATION_TIME)).thenReturn(expectedValue);

				var kopf = createKopf();

				assertThat(kopf.getErstellungszeitpunkt()).isEqualTo(expectedValue);
			}
		}

		@Test
		void shouldSetAbsender() {
			var expectedValue = new KontaktType();
			doReturn(expectedValue).when(creator).createAbsender(any());

			var kopf = createKopf();

			assertThat(kopf.getAbsender()).isEqualTo(expectedValue);
		}

		@Test
		void shouldSetEmpfaenger() {
			var expectedValue = new KontaktType();
			doReturn(expectedValue).when(creator).createKontaktType();

			var kopf = createKopf();

			assertThat(kopf.getEmpfaenger()).isEqualTo(expectedValue);
		}

		@Test
		void shouldSetSendendesSystem() {
			var sendendesSystem = new SystemType();
			doReturn(sendendesSystem).when(creator).creeateSendendesSystem();

			var kopf = createKopf();

			assertThat(kopf.getSendendesSystem()).isEqualTo(sendendesSystem);
		}

		@Test
		void shouldSetImportbestaetigung() {
			var kopf = createKopf();

			assertThat(kopf.isImportbestaetigung()).isTrue();
		}

		@Test
		void shouldSetEmpfangsbestaetigung() {
			var kopf = createKopf();

			assertThat(kopf.isEmpfangsbestaetigung()).isTrue();
		}

		private NkAbgabeType createKopf() {
			return creator.createKopf(VorgangWithEingangTestFactory.create());
		}
	}

	@Nested
	class TestCreateNachrichtentype {

		@Test
		void shouldSetCode() {
			var nachrichtentyp = creator.createNachrichtentyp();

			assertThat(nachrichtentyp.getCode()).isEqualTo(KopfCreator.NACHRICHTENTYP_ABGABE_ABGABE_TYPE_CODE);
		}

		@Test
		void shouldSetListURI() {
			var nachrichtentyp = creator.createNachrichtentyp();

			assertThat(nachrichtentyp.getListURI()).isEqualTo(KopfCreator.NACHRICHTENTYP_CODE_TYPE_LIST_URI);
		}

		@Test
		void shouldSetListVersionID() {
			var nachrichtentyp = creator.createNachrichtentyp();

			assertThat(nachrichtentyp.getListVersionID()).isEqualTo(KopfCreator.NACHRICHTENTYP_CODE_TYPE_LIST_VERSION_ID);
		}
	}

	@Nested
	class TestCreateAbsender {

		@Mock
		private BehoerdenkennungType behoerdenkennungType;

		@Test
		void shouldCallCreateKontaktType() {
			creator.createAbsender(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			verify(creator).createKontaktType();
		}

		@Test
		void shouldSetOrganisationseinheitType() {
			var expectedValue = new OrganisationseinheitType();
			doReturn(expectedValue).when(creator).createOrganisationseinheitType(any());

			var absender = creator.createAbsender(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			assertThat(absender.getOrganisationseinheit()).isEqualTo(expectedValue);
		}
	}

	@Nested
	class TestCreateKontaktType {

		@Test
		void shouldSetBehoerdenkennungType() {
			var expectedValue = new BehoerdenkennungType();
			doReturn(expectedValue).when(creator).createBehoerdenkennung();

			var kontaktType = creator.createAbsender(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			assertThat(kontaktType.getBehoerdenkennung()).isEqualTo(expectedValue);

		}
	}

	@Nested
	class TestCreateOrganisationseinheitType {

		@Test
		void shouldSetName() {
			var organisationseinheitType = creator.createOrganisationseinheitType(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			assertThat(organisationseinheitType.getName()).isEqualTo(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);
		}
	}

	@Nested
	class TestCreateBehoerdenkennung {

		@Test
		void shouldSetBehoerdenschluessel() {
			var expectedBehoerdenschluessel = new Code();
			doReturn(expectedBehoerdenschluessel).when(creator).createBehoerdenschlussen();

			var behoerdenkennungType = creator.createBehoerdenkennung();

			assertThat(behoerdenkennungType.getBehoerdenschluessel()).isEqualTo(expectedBehoerdenschluessel);
		}
	}

	@Nested
	class TestCreateBehoerdenschlussel {

		@Test
		void shouldSetCode() {
			var expectedBehoerdenschluessel = "123456789";
			when(xDomeaProperties.getBehoerdenschluessel()).thenReturn(expectedBehoerdenschluessel);

			var behoerdenschlussel = creator.createBehoerdenschlussen();

			assertThat(behoerdenschlussel.getCode()).isEqualTo(expectedBehoerdenschluessel);
		}

		@Test
		void shouldSetListURI() {
			var behoerdenschlussel = creator.createBehoerdenschlussen();

			assertThat(behoerdenschlussel.getListURI()).isEqualTo(KopfCreator.BEHOERDENSCHLUSSEL_LIST_URI);
		}

		@Test
		void shouldSetListVersionID() {
			var behoerdenschlussel = creator.createBehoerdenschlussen();

			assertThat(behoerdenschlussel.getListVersionID()).isEqualTo(KopfCreator.BEHOERDENSCHLUSSEL_LIST_VERSION_ID);
		}
	}

	@Nested
	class TestCreateSendendesSystem {

		@Test
		void shouldSetSystemName() {
			var sendendesSystem = creator.creeateSendendesSystem();

			assertThat(sendendesSystem.getProduktname()).isEqualTo(KopfCreator.PRODUKT_NAME);
		}
	}
}